"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const react_usestateref_1 = __importDefault(require("react-usestateref"));
const shallow_equal_1 = require("shallow-equal");
const use_latest_callback_1 = __importDefault(require("use-latest-callback"));
const usehooks_ts_1 = require("usehooks-ts");
const useAsyncHandler_1 = __importDefault(require("./useAsyncHandler"));
function useAsyncFetch(onActionFn, options, dependencies) {
    if (options instanceof Array) {
        dependencies = options;
        options = {};
    }
    if (!dependencies)
        dependencies = [];
    (0, react_1.useEffect)(() => {
        if (dependencies)
            dependencies.forEach((dep) => {
                if (dep && typeof dep === 'object') {
                    console.warn('[useAsyncFetch]', 'Try to avoid using objects in dependencies. It might cause recursive runs. Prefer primitive types.');
                }
            });
    }, dependencies);
    const { maxTries = 1, timeoutBeforeRetry = 1000 } = options || {};
    const [tries, setTries, triesRef] = (0, react_usestateref_1.default)(0);
    const [isInRetryTimeout, setRetryTimeoutEnabled] = (0, react_usestateref_1.default)(false);
    const asyncAction = (0, useAsyncHandler_1.default)(onActionFn, { strict: true });
    const { isLoading, isDone, isErrored, error, data, indicators } = asyncAction;
    const depsPrevRef = (0, react_1.useRef)();
    const handleTry = (0, use_latest_callback_1.default)(function handleTryFn() {
        const { isLoading, isDone } = indicators.stateRef.current;
        const tries = triesRef.current || 0;
        if (isDone || isLoading)
            return;
        depsPrevRef.current = dependencies;
        setTries(tries + 1);
        asyncAction.execute().catch((e) => {
            let actionPrefix = onActionFn.name ? `(${onActionFn.name}) ` : '';
            console.error(`${actionPrefix}AsyncFetch action got error:`);
            console.error(e);
        });
    });
    const reset = (0, use_latest_callback_1.default)(function resetFn() {
        asyncAction.reset();
        setTries(0);
        handleTry();
    });
    (0, react_1.useEffect)(() => {
        const isEqual = depsPrevRef.current ? (0, shallow_equal_1.shallowEqualArrays)(depsPrevRef.current, dependencies) : false;
        if (isLoading || isEqual) {
            return;
        }
        reset();
    }, [isLoading, ...dependencies]);
    const isFetchNecessary = !isLoading && !isDone;
    (0, react_1.useEffect)(() => {
        if (isInRetryTimeout)
            return;
        if (!isFetchNecessary)
            return;
        if (!tries || tries >= maxTries)
            return;
        setRetryTimeoutEnabled(true);
    }, [isFetchNecessary, tries]);
    function handleTimeout() {
        setRetryTimeoutEnabled(false);
        handleTry();
    }
    (0, usehooks_ts_1.useTimeout)(handleTimeout, isInRetryTimeout ? timeoutBeforeRetry : null);
    return (0, react_1.useMemo)(() => ({
        isLoading,
        isDone,
        isErrored,
        isInRetryTimeout,
        error,
        data,
        reset,
        tries,
        indicators,
    }), [asyncAction, isInRetryTimeout, tries]);
}
exports.default = useAsyncFetch;
//# sourceMappingURL=useAsyncFetch.js.map